from django.db import models
from django.template.loader import render_to_string


class InlineEditableField:

    def __init__(self, can_edit, image_field_kwargs=None, text_field_kwargs=None):
        self.image_field_kwargs = image_field_kwargs or {}
        self.text_field_kwargs = text_field_kwargs or {}

        if not callable(can_edit):
            raise TypeError(
                'The can_edit parameter needs to be a function that takes one parameter, the current request.'
            )

        self.can_edit = can_edit

    def __get__(self, instance, owner):
        if instance:
            self.obj_pk = instance.pk
            self.obj = instance
        return self

    def contribute_to_class(self, cls, name):
        self.model = cls.__name__
        self.app = cls._meta.app_label
        self.name = name

        if not getattr(cls, '_has_image_model', False):
            model = type(
                "{}_Images".format(cls.__name__),
                (models.Model,),
                {
                    "__module__": cls.__module__,
                    "owner": models.ForeignKey(cls, on_delete=models.CASCADE),
                    "image": models.ImageField(**self.image_field_kwargs),

                }
            )
            cls.image_model = model
            cls._has_image_model = True
        else:
            model = cls.image_model

        setattr(cls, '{}_images'.format(name), model.objects.values_list("image", flat=True))
        setattr(cls, name, self)

        text_field = models.TextField(**self.text_field_kwargs)
        text_field.set_attributes_from_name('{}_text'.format(name))
        text_field.model = cls
        cls._meta.add_field(text_field)

        return model

    @property
    def rendered(self):
        context = {
            'field': self,
            'editor_selector': "{}-{}-{}".format(self.app, self.model, self.name),
            'field_content': getattr(self.obj, '{}_text'.format(self.name))
        }
        return render_to_string('editor.html', context=context)



import os.path
import uuid

from django.apps import apps
from django.core.files.base import ContentFile
from django.http import HttpResponse
from django.http import HttpResponseNotAllowed
from django.views.generic import View


class IEFTextHandler(View):

    def post(self, request, app, model, field, pk):
        model = apps.get_model(app, model)
        if getattr(model, field).can_edit(request):
            obj = model.objects.get(pk=pk)
            setattr(obj, '{}_text'.format(field), request.read().decode())
            obj.save()
            return HttpResponse()
        else:
            raise HttpResponseNotAllowed


class IEFImageHandler(View):

    def post(self, request, app, model, field, pk):
        model = apps.get_model(app, model)
        if getattr(model, field).can_edit(request):
            img = ContentFile(request.body)
            image_obj = model.image_model(owner_id=pk)
            field_upload_to = getattr(model, field).image_field_kwargs.get('upload_to', '')
            image_upload_path = os.path.join(field_upload_to, "file:{}.jpg".format(uuid.uuid4()))

            image_obj.image.save(image_upload_path, img)
            image_obj.save()
            return HttpResponse(image_obj.image.url)
        else:
            raise HttpResponseNotAllowed



from setuptools import setup

with open('README.md') as f:
    long_description = f.read()

setup(
    name='django_ief',
    version='0.1.1',
    packages=['django_ief', 'django_ief.templatetags'],
    include_package_data=True,
    url='https://github.com/isik-kaplan/django_ief',
    description="A model field that can be edited without needing a form.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='APGL-3.0',
    author='isik-kaplan',
    author_email='',
    python_requires=">=3.5",
    install_requires=['django>=2.0']
)



from django.urls import path

from .views import IEFTextHandler, IEFImageHandler

urlpatterns = [
    path('text/<str:app>/<str:model>/<str:field>/<int:pk>/', IEFTextHandler.as_view(), name="inline_editable_field_text"),
    path('image/<str:app>/<str:model>/<str:field>/<int:pk>/', IEFImageHandler.as_view(), name="inline_editable_field_image"),
]
